*==============================================================================
* Program:				QueryWithBuffering.PRG
* Purpose:				Demonstrate the use of the WITH (Buffering = .T.)
*                       clause by modifying buffered data, then
*                       querying against it.
* Author:				Tamar E. Granor
* Copyright:			(c) 2004 Tamar E. Granor
* Last revision:		6/24/2004
* Parameters:			None
* Returns:				None
* Environment in:		None
* Environment out:		None
*==============================================================================

LOCAL cOldMultiLocks

cOldMultiLocks = SET("Multilocks")
SET MULTILOCKS ON

* Open Customer table and turn on 
* table buffering
OPEN DATABASE HOME(2) + "TasTrade\Data\TasTrade"
SELECT 0
USE Customer
CURSORSETPROP("Buffering", 5)

* Compute original count by country
SELECT Country, CNT(*) ;
	FROM Customer ;
	GROUP BY Country ;
	INTO CURSOR OriginalCount

* Display results
MESSAGEBOX("Customer count by country before any changes. " + ;
           "Be sure to check Austria and Germany." )
BROWSE  WINDOW ResultWin  
           
* Change a record in the buffer
REPLACE Country WITH "Austria" IN Customer

* Compute count by country without
* specifying buffering
SELECT Country, CNT(*) ;
	FROM Customer ;
	GROUP BY Country ;
	INTO CURSOR AgainstDisk

* Display results
MESSAGEBOX("Customer count by country after change. " + ;
           "Query does not include WITH Buffering clause. " + ;
           "Be sure to check Austria and Germany." )
BROWSE  WINDOW ResultWin          

* Compute count by country with
* buffering
SELECT Country, CNT(*) ;
	FROM Customer WITH (Buffering = .T.);
	GROUP BY Country ;
	INTO CURSOR BufferedCount
	
* Display results
MESSAGEBOX("Customer count by country after change. " + ;
           "Query includes WITH Buffering clause. " + ;
           "Be sure to check Austria and Germany." )
BROWSE WINDOW ResultWin           

* Clean up
SELECT Customer
TABLEREVERT(.T.)

USE IN Customer
USE IN OriginalCount
USE IN AgainstDisk
USE IN BufferedCount
SET MULTILOCKS &cOldMultiLocks
