*==============================================================================
* Program:				WAREHOUSEUNION.PRG
* Purpose:				Add a year's worth of information to a data warehouse
*                       showing sales of products by employees.
* Author:				Tamar E. Granor
* Copyright:			(c) 2004 Tamar E. Granor
* Last revision:		6/24/2004
* Parameters:			None
* Returns:				None
* Environment in:		TasTrade database open,
*                       variable nYear populated
* Environment out:		Cursor Warehouse created and populated with 
*                       data for specified year.
*==============================================================================

***************************************
* Make sure to create and set nYear 
* before running this code
nYear = 1994
***************************************

CREATE CURSOR Warehouse (Product_ID C(6), Employee_ID C(6), ;
                  nYear N(4), ;
                  nTotalSales Y, nUnitsSold N(8))

INSERT INTO Warehouse ;
SELECT CrossProd.Product_ID, ;
       CrossProd.Employee_ID, ;
       m.nYear as nYear, ;
       NVL(nUnitsSold, 0), NVL(nTotalSales, $0);
	FROM (SELECT Employee.Employee_ID, Products.Product_ID ;
		FROM Employee, Products) AS CrossProd ;
	  LEFT JOIN ( ;
		SELECT Product_ID, Employee_ID, ;
		       SUM(Quantity) AS nUnitsSold, ;
		       SUM(Quantity * Unit_Price) AS nTotalSales ;
			FROM Orders ;
			  JOIN Order_Line_Items ;
			    ON Orders.Order_ID = Order_Line_Items.Order_ID ;
			WHERE YEAR(Order_Date) = m.nYear ;
			GROUP BY Product_ID, Employee_ID ) AS AnnualSales ;
		ON CrossProd.Employee_ID = AnnualSales.Employee_ID  ;
		AND CrossProd.Product_ID = AnnualSales.Product_ID ;
UNION ;
SELECT "Total" AS Product_ID, Employee.Employee_ID, ;
       m.nYear AS nYear, ;
       CAST(NVL(SUM(Quantity),0) as N(12)) AS nUnitsSold, ;
       NVL(SUM(Quantity * Unit_Price), $0) AS nTotalSales ;
	FROM Orders ;
	  JOIN Order_Line_Items ;
	    ON Orders.Order_ID = Order_Line_Items.Order_ID ; 
		AND YEAR(Order_Date) = m.nYear ;
	  RIGHT JOIN Employee ;
	    ON Orders.Employee_ID = Employee.Employee_ID ;
	GROUP BY Employee.Employee_ID ;
ORDER BY 2, 1

BROWSE WINDOW ResultWin